#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <assert.h>


#include "Types.h"
#include "encoder.h"
#include "common.h"
#include "tonal.h"
#include "BladeDll.h"

#define ADD_ANCILLARY_DATA 1

#ifdef _DEBUG
	//#define DEBUG_ENCODER
#endif


#ifdef ADD_ANCILLARY_DATA
	static int	nMaxLeftEnergy=0;
	static int	nMaxRightEnergy=0;
#endif

#define MAX_STREAMS 1


//BE_CONFIG		StreamConfig[MAX_STREAMS+1];
//char original_file_name[MAX_NAME_SIZE];
//char encoded_file_name[MAX_NAME_SIZE];
//static short buffer[2][1152];


// Local function prototypes
static BE_ERR DeInitEncoder();
static BE_ERR InitEncoder();


// LOCAL TYPE DEFINITIONS
typedef double	SBS[2][3][SCALE_BLOCK][SBLIMIT];
typedef double	JSBS[3][SCALE_BLOCK][SBLIMIT];
typedef UINT	SUB[2][3][SCALE_BLOCK][SBLIMIT];
typedef double	MYIN[2][HAN_SIZE];

typedef struct GLOBALVARS_TAG
{
	Bit_stream_struc   bs;
#ifdef LAYER2
	SBS*			sb_sample;
#endif
#ifdef LAYER3
    L3SBS*			l3_sb_sample;
#endif
	frame_params	fr_ps;
	JSBS*			j_sample;
//	MYIN*			win_que;
	layer			info;
	short**			win_buf;
	SUB*			subband;
	UINT			bit_alloc[2][SBLIMIT];
	UINT			scfsi[2][SBLIMIT];
	UINT			scalar[2][3][SBLIMIT];
	UINT			j_scale[3][SBLIMIT];
	double			ltmin[2][SBLIMIT];
	double			lgmin[2][SBLIMIT];
	double			max_sc[2][SBLIMIT];
	FLOAT			snr32[32];
	short			sam[2][1344]; /* was [1056]; */

	int				j52ancillaryzeros;	// Added by Marko Vitez for J.52 bitrates
	int				bitrate;			// Added by Marko Vitez for J.52 bitrates
	int				whole_SpF;
	int				extra_slot;
	int				model;
	double			avg_slots_per_frame;
	double			frac_SpF;
	double			slot_lag;

	UINT			crc;
//	int i, j, k, adb;
	ULONG			bitsPerSlot;
	ULONG			samplesPerFrame;
	ULONG			frameBits;
	ULONG			sentBits;
	ULONG			num_samples;

	INT				adb;
    short			buffer[2][MPEGFRAMESIZE];
} GLOBALVARS;



// Define global variables
int					qsfactor = 0;		// quality speed factor used in loop.c
int					swapbytes = FALSE;	// force byte swapping
FILE*				musicin;
//Bit_stream_struc   bs;
char*				programName;
int					iswav=0;
int					autoconvert=FALSE;
int					resample=FALSE;
static int			nopadding;


// Define static global variables
static GLOBALVARS glb;


static BE_ERR InitEncoder()
{
    // Allocate dynamic memory
#ifdef LAYER2
	glb.sb_sample		=(SBS*)mem_alloc(sizeof(SBS),"glb.sb_sample");
#endif
#ifdef LAYER3
    glb.l3_sb_sample	=(L3SBS*)mem_alloc(sizeof(L3SBS), "glb.l3_sb_sample");
#endif
    glb.j_sample		=(JSBS*)mem_alloc(sizeof(JSBS), "glb.j_sample");
//    glb.win_que			=NULL;
    glb.subband			=(SUB*)mem_alloc(sizeof(SUB),"glb.subband");
    glb.win_buf			=(short**) mem_alloc(sizeof(short *)*2, "win_buf");

	// Set non zero variables 
    glb.fr_ps.header = &glb.info;

    // no table loaded
	glb.fr_ps.tab_num = -1;

    glb.fr_ps.alloc = NULL;

	// Default: MPEG-1
    glb.fr_ps.header->version = MPEG_AUDIO_ID;

#ifdef LAYER1    
    if (glb.fr_ps.header->lay == 1)
    { 
		glb.bitsPerSlot = 32; glb.samplesPerFrame = 384;  
	}
#endif
#ifdef LAYER2
	if ( glb.fr_ps.header->lay == 2 )
	{ 
		glb.bitsPerSlot = 8;  glb.samplesPerFrame = 1152; 
	}
#endif
#ifdef LAYER3
	{  /* layer 3 */
	    glb.bitsPerSlot = 8;
	    glb.samplesPerFrame = glb.fr_ps.header->version == 1 ? 1152 : 576;
	    
	    // Apologize for missing features
	    if ( glb.fr_ps.header->mode == MPG_MD_JOINT_STEREO )
	    {
			strcpy(lpszError,"Sorry, joint stereo not yet available for layer3\n" );
			MPegError(lpszError);
	    }

	    if ( glb.model != 2 )
	    {
			glb.model=2;
	    }
	}
#endif
    /* Figure average number of 'slots' per frame. */
    /* Bitrate means TOTAL for both channels, not per side. */
    glb.avg_slots_per_frame = (double)glb.samplesPerFrame;
	glb.avg_slots_per_frame /=(double)s_freq[glb.fr_ps.header->version][glb.fr_ps.header->sampling_frequency];
	glb.avg_slots_per_frame *=(double)glb.bitrate / 1000.0; //bitrate[glb.fr_ps.header->version][glb.fr_ps.header->lay-1][glb.fr_ps.header->bitrate_index];
	glb.avg_slots_per_frame /=(double)glb.bitsPerSlot;
    glb.whole_SpF = (int) glb.avg_slots_per_frame;

	// Added by Marko Vitez for J.52 bitrates
	glb.j52ancillaryzeros = (int)(
		(double)glb.samplesPerFrame
		/ (double)s_freq[glb.fr_ps.header->version][glb.fr_ps.header->sampling_frequency]
		* (double)bitrate[glb.fr_ps.header->version][glb.fr_ps.header->lay-1][glb.fr_ps.header->bitrate_index]
		/ (double)glb.bitsPerSlot) - glb.whole_SpF;

    //printf("j52ancillaryzeros = %d\n",glb.j52ancillaryzeros);
    //printf("slots/frame = %d\n",glb.whole_SpF);
    glb.frac_SpF  = glb.avg_slots_per_frame - (double)glb.whole_SpF;
	// Added by Marko Vitez for padding
	if(nopadding)
		glb.frac_SpF = 0;
    glb.slot_lag  = -glb.frac_SpF;

    //printf("frac SpF=%.3f, tot bitrate=%d kbps, s freq=%.1f kHz\n",
//           glb.frac_SpF, bitrate[glb.fr_ps.header->version][glb.fr_ps.header->lay-1][glb.fr_ps.header->bitrate_index],
  //         s_freq[glb.fr_ps.header->version][glb.fr_ps.header->sampling_frequency]);
    
    if (glb.frac_SpF != 0)
	{
	//	printf("Fractional number of slots, padding required\n");
	}
    else
	{
		glb.fr_ps.header->padding = 0;
	}

	open_bit_stream_w(&glb.bs, "C:\\temp.mp3", BUFFER_SIZE/2);

	return BE_ERR_SUCCESSFUL;
}


static BE_ERR DeInitEncoder()
{
	#ifdef LAYER_3
		if ( glb.fr_ps.header->lay == 3 )
			III_FlushBitstream();
	#endif
		close_bit_stream_w( &glb.bs );

	#ifdef LAYER3
		free(glb.l3_sb_sample);
	#endif
	#ifdef LAYER2
		free (glb.sb_sample);
	#endif
	free(glb.j_sample);
//	free(glb.win_que);
	free(glb.subband);
	free(glb.win_buf);

	return BE_ERR_SUCCESSFUL;
}




__declspec(dllexport) BE_ERR	beInitStream(PBE_CONFIG pbeConfig, PDWORD dwSamples, PDWORD dwBufferSize, PHBE_STREAM phbeStream)
{
//	int* pTest=NULL;

	// Initialize the encoder
	BE_ERR funcResult;

	// Clear all global variables
	memset(&glb,0x00,sizeof(glb));

	// Set non zero variables 
    glb.fr_ps.header = &glb.info;


	// For now always assign stream 0
	*phbeStream=1;

	// Copy configuration paramters
	//StreamConfig[*phbeStream]=pbeConfig,sizeof(BE_CONFIG));

	// Set MP3 buffer size
	*dwBufferSize=BUFFER_SIZE;

	// Set number of input samples depending on the number of samples
	if (pbeConfig->format.mp3.byMode== BE_MP3_MODE_MONO)
	{
		// Mono channel, thus MPEGFRAMSIZE samples needed
		*dwSamples=MPEGFRAMESIZE;
	}
	else
	{
		// For Stereo.Joint stereo and dual channel, need 2*MPEGFRAMESIZE samples
		*dwSamples=MPEGFRAMESIZE*2;
	}

//	info->copyright			= 0; 
//	info->original			= 0; 

	// Setup MP3 paramters
    glb.fr_ps.header->emphasis=0;
    glb.fr_ps.header->mode_ext=0;

//    glb.fr_ps.header.extension;
    glb.fr_ps.header->mode=pbeConfig->format.mp3.byMode;
    glb.fr_ps.header->copyright=pbeConfig->format.mp3.bCopyright;
    glb.fr_ps.header->original=pbeConfig->format.mp3.bOriginal;
	glb.fr_ps.header->error_protection= pbeConfig->format.mp3.bCRC;
//			BOOL	bPrivate;		

// Set layer depending on encoder type
#ifdef LAYER2
	glb.fr_ps.header->lay = 2;
#else
	glb.fr_ps.header->lay = 3;
#endif

	// Set MPEG version to detault
	glb.fr_ps.header->version=MPEG_AUDIO_ID;

	// Set sample frequency of input stream
	if((glb.fr_ps.header->sampling_frequency = SmpFrqIndex(pbeConfig->format.mp3.dwSampleRate,&glb.fr_ps.header->version)) < 0) 
	{
		return BE_ERR_INVALID_FORMAT_PARAMETERS;
	}

	// Added by Marko Vitez for J.52 bitrates
	if(pbeConfig->format.mp3.wBitrate == 62)
	{
		glb.fr_ps.header->bitrate_index=BitrateIndex(glb.fr_ps.header->lay,64,glb.fr_ps.header->version);
		glb.bitrate = 62400;
	} else if(pbeConfig->format.mp3.wBitrate == 124)
	{
		glb.fr_ps.header->bitrate_index=BitrateIndex(glb.fr_ps.header->lay,128,glb.fr_ps.header->version);
		glb.bitrate = 124800;
	} else {
		// Set bitrate index depending on the layer
		glb.fr_ps.header->bitrate_index=BitrateIndex(glb.fr_ps.header->lay,pbeConfig->format.mp3.wBitrate,glb.fr_ps.header->version);
		glb.bitrate = pbeConfig->format.mp3.wBitrate * 1000;
	}

	nopadding = glb.bitrate != 62400 && glb.bitrate != 124800;
	if ( (funcResult=InitEncoder()) !=BE_ERR_SUCCESSFUL)
		return funcResult;

	// Set header parameters    
    hdr_to_frps(&glb.fr_ps);

//*pTest=0;

//	if (glb.fr_ps.header->version!=MPEG_AUDIO_ID)
//	{
//		only need half the samples for MPEG Layer-2
//	}
#ifdef ADD_ANCILLARY_DATA
	nMaxLeftEnergy=0;
	nMaxRightEnergy=0;
#endif

	// Everything went OK, thus return SUCCESSFUL
	return BE_ERR_SUCCESSFUL;
}



__declspec(dllexport) BE_ERR	beDeinitStream(HBE_STREAM hbeStream, PBYTE pOutput, PDWORD pdwOutput)
{
	// Initialize output buffer
	glb.bs.pbtOutBuf=pOutput;
	glb.bs.nOutBufPos=0;

	// Clear buffer memory
	memset(&glb.buffer[0][0],0x00,2*sizeof(SHORT)*MPEGFRAMESIZE);

	// Flush last bytes
   empty_buffer(&glb.bs,glb.bs.buf_byte_idx);

   // Number of bytes in output buffer
	*pdwOutput=glb.bs.nOutBufPos;

	if (*pdwOutput>0)
		*pdwOutput=*pdwOutput-1;

	// Copy the output buffer
	memcpy(pOutput,glb.bs.pbtOutBuf,*pdwOutput);
	
	// Everything went OK, thus return SUCCESSFUL
	return BE_ERR_SUCCESSFUL;
}


__declspec(dllexport) BE_ERR	beCloseStream(HBE_STREAM hbeStream)
{
	// DeInit encoder
	return DeInitEncoder();
}



__declspec(dllexport) VOID		beVersion(PBE_VERSION pbeVersion)
{
	// DLL Release date
	char lpszDate[20];
	char lpszTemp[5];


	// Set Major version number
	pbeVersion->byMajorVersion=pbeVersion->byDLLMajorVersion=1;

	// Set minor version number
	pbeVersion->byMinorVersion=pbeVersion->byDLLMinorVersion=11;


	// Get compilation date
	strcpy(lpszDate,__DATE__);

	// Get the first three character, which is the month
	strncpy(lpszTemp,lpszDate,2);

	// Set month
	if (strcmp(lpszTemp,"Jan")==0)	pbeVersion->byMonth=1;
	if (strcmp(lpszTemp,"Feb")==0)	pbeVersion->byMonth=2;
	if (strcmp(lpszTemp,"Mar")==0)	pbeVersion->byMonth=3;
	if (strcmp(lpszTemp,"Apr")==0)	pbeVersion->byMonth=4;
	if (strcmp(lpszTemp,"May")==0)	pbeVersion->byMonth=5;
	if (strcmp(lpszTemp,"Jun")==0)	pbeVersion->byMonth=6;
	if (strcmp(lpszTemp,"Jul")==0)	pbeVersion->byMonth=7;
	if (strcmp(lpszTemp,"Aug")==0)	pbeVersion->byMonth=8;
	if (strcmp(lpszTemp,"Sep")==0)	pbeVersion->byMonth=9;
	if (strcmp(lpszTemp,"Oct")==0)	pbeVersion->byMonth=10;
	if (strcmp(lpszTemp,"Nov")==0)	pbeVersion->byMonth=11;
	if (strcmp(lpszTemp,"Dec")==0)	pbeVersion->byMonth=12;

	// Get day of month string (char [3..4])
	strncpy(lpszTemp,lpszDate+3,2);
	// Set day of compilation date
	pbeVersion->byDay=atoi(lpszTemp);

	// Get year of compilation date (char [5..8])
	strncpy(lpszTemp,lpszDate+5,4);

	// Set year of compilation date
	pbeVersion->wYear=atoi(lpszTemp);
	pbeVersion->wYear=1999;

	memset(pbeVersion->zHomepage,0x00,BE_MAX_HOMEPAGE);

	strcpy(pbeVersion->zHomepage,"http://www.cdex.n3.net");
}



__declspec(dllexport) BE_ERR	beEncodeChunk(HBE_STREAM hbeStream, DWORD nSamples, PSHORT pSamples, PBYTE pOutput, PDWORD pdwOutput)
{
//	fprintf(stderr, "{%4lu}\r", frameNum++); fflush(stderr);
	int				i,j,k=0;

//	SHORT* win_buf[2];


	// Initialize output buffer
	glb.bs.pbtOutBuf=pOutput;
	glb.bs.nOutBufPos=0;

	glb.win_buf[0] = &glb.buffer[0][0];
	glb.win_buf[1] = &glb.buffer[1][0];

	// Clear buffer memory
	memset(&glb.buffer[0][0],0x00,2*sizeof(SHORT)*MPEGFRAMESIZE);


	// Copy data depending on the number of channels
	if (glb.fr_ps.stereo==1)
	{
		for (i=0;i<(int)nSamples;i++)
		{
			glb.buffer[0][i] = *pSamples++;
			#ifdef ADD_ANCILLARY_DATA
				nMaxLeftEnergy=max(nMaxLeftEnergy,glb.buffer[0][i]);
			#endif
		}
	}
	else
	{
		for (i=0;i<(int)nSamples/2;i++)
		{
			glb.buffer[0][i] = *pSamples++;
			glb.buffer[1][i] = *pSamples++;
			#ifdef ADD_ANCILLARY_DATA
				nMaxLeftEnergy=max(nMaxLeftEnergy,glb.buffer[0][i]);
				nMaxRightEnergy=max(nMaxRightEnergy,glb.buffer[1][i]);
			#endif
		}
	}


	if (glb.frac_SpF != 0)
	{
		if (glb.slot_lag > (glb.frac_SpF-1.0) )
		{
			glb.slot_lag -= glb.frac_SpF;
			glb.extra_slot = 0;
			glb.fr_ps.header->padding = 0;
			//  printf("No padding for this frame\n");
		}
		else
		{
			glb.extra_slot = 1;
			glb.fr_ps.header->padding = 1;
			glb.slot_lag += (1-glb.frac_SpF);
			//  printf("Padding for this frame\n");
		}
	}

	glb.adb = (glb.whole_SpF+glb.extra_slot) * glb.bitsPerSlot;

#ifdef ADD_ANCILLARY_DATA
	// For J.52 bitrates there is no additional ancillary data
	if(glb.bitrate != 124800 && glb.bitrate != 62400)
	{
		if (glb.fr_ps.stereo==1)
			glb.adb-=3*8;
		else
			glb.adb-=5*8;
	}
#endif
	
	switch (glb.fr_ps.header->lay)
	{
/***************************** Layer I **********************************/
          case 1 :
#ifndef LAYER1
			MPegError("Layer 1 not supported");
#else
	    for (j=0;j<SCALE_BLOCK;j++)
		{
//			for (k=0;k<glb.fr_ps.stereo;k++)
//			{
//			    window_subband(&win_buf[k], &(*glb.win_que)[k][0], k);
//			    filter_subband(&(*glb.win_que)[k][0], &(*glb.sb_sample)[k][0][j][0]);
//			}
			window_and_filter_subband(&win_buf[0],0,&(*glb.sb_sample)[0][0][j][0]);
			if (glb.fr_ps.stereo>1)
				window_and_filter_subband(&win_buf[1],1,&(*glb.sb_sample)[1][0][j][0]);
		}
	    
	    I_scale_factor_calc(*glb.sb_sample, glb.scalar, glb.fr_ps.stereo);
	    if(glb.fr_ps.actual_mode == MPG_MD_JOINT_STEREO) {
                I_combine_LR(*glb.sb_sample, *glb.j_sample);
                I_scale_factor_calc(glb.j_sample, &glb.j_scale, 1);
	    }
	    
	    put_scale(glb.scalar, &glb.fr_ps, glb.max_sc);
	    
	    if (glb.model == 1) I_Psycho_One(buffer, glb.max_sc, glb.ltmin, &glb.fr_ps);
	    else {
                for (k=0;k<glb.fr_ps.stereo;k++) {
		    psycho_anal(&buffer[k][0],&glb.sam[k][0], k, glb.fr_ps.header->lay, glb.snr32,
				(FLOAT)s_freq[glb.fr_ps.header->version][glb.fr_ps.header->sampling_frequency]*1000);
		    for (i=0;i<SBLIMIT;i++) glb.ltmin[k][i] = (double) glb.snr32[i];
                }
	    }
	    
	    I_main_bit_allocation(glb.ltmin, glb.bit_alloc, &adb, &glb.fr_ps);
	    
	    if (glb.fr_ps.header->error_protection)
			I_CRC_calc(&glb.fr_ps, glb.bit_alloc, &glb.crc);
	    
	    encode_info(&glb.fr_ps, &bs);
	    
	    if (glb.fr_ps.header->error_protection)
			encode_CRC(glb.crc, &bs);
	    
	    I_encode_bit_alloc(glb.bit_alloc, &glb.fr_ps, &bs);
	    I_encode_scale(glb.scalar, glb.bit_alloc, &glb.fr_ps, &bs);
	    I_subband_quantization(glb.scalar, *glb.sb_sample, glb.j_scale, *glb.j_sample,
				   glb.bit_alloc, *glb.subband, &glb.fr_ps);
	    I_sample_encoding(*glb.subband, glb.bit_alloc, &glb.fr_ps, &bs);
	    for (i=0;i<adb;i++) put1bit(&bs, 0);
#endif
	    break;
	    
/***************************** Layer 2 **********************************/
	    
          case 2 :
#ifndef LAYER2
			MPegError("MPEG Layer 2 not supported\n");
#else
	    for (i=0;i<3;i++)
		{
			for (j=0;j<SCALE_BLOCK;j++)
			{
//				for (k=0;k<glb.fr_ps.stereo;k++) 
//				{
//					window_subband(&glb.win_buf[k], &(*glb.win_que)[k][0], k);
//					filter_subband(&(*glb.win_que)[k][0], &(*glb.sb_sample)[k][i][j][0]);
//				}
				window_and_filter_subband(&glb.win_buf[0],0,&(*glb.sb_sample)[0][i][j][0]);
				if (glb.fr_ps.stereo>1)
					window_and_filter_subband(&glb.win_buf[1],1,&(*glb.sb_sample)[1][i][j][0]);

			}
		}
	
	    II_scale_factor_calc(*glb.sb_sample, glb.scalar, glb.fr_ps.stereo, glb.fr_ps.sblimit);
	    pick_scale(glb.scalar, &glb.fr_ps, glb.max_sc);

		if(glb.fr_ps.actual_mode == MPG_MD_JOINT_STEREO)
		{
			II_combine_LR(*glb.sb_sample, *glb.j_sample, glb.fr_ps.sblimit);
			II_scale_factor_calc(glb.j_sample, &glb.j_scale, 1, glb.fr_ps.sblimit);
	    }       // this way we calculate more mono than we need but it is cheap
	    
	    if (glb.model == 1)
			II_Psycho_One(glb.buffer, glb.max_sc, glb.ltmin, &glb.fr_ps);
	    else
		{
			for (k=0;k<glb.fr_ps.stereo;k++)
			{
				psycho_anal(	&glb.buffer[k][0],
								&glb.sam[k][0],
								k, 
								glb.fr_ps.header->lay, glb.snr32,
								(FLOAT)s_freq[glb.fr_ps.header->version][glb.fr_ps.header->sampling_frequency]*1000
							);

				for (i=0;i<SBLIMIT;i++)
					glb.ltmin[k][i] = (double) glb.snr32[i];
			}
	    }
	    
	    II_transmission_pattern(glb.scalar, glb.scfsi, &glb.fr_ps);
	    II_main_bit_allocation(glb.ltmin, glb.scfsi, glb.bit_alloc, &glb.adb, &glb.fr_ps);
	    
	    if (glb.fr_ps.header->error_protection)
			II_CRC_calc(&glb.fr_ps, glb.bit_alloc, glb.scfsi, &glb.crc);
	    
		encode_info(&glb.fr_ps, &glb.bs);
	    
	    if (glb.fr_ps.header->error_protection)
			encode_CRC(glb.crc, &glb.bs);
	    
	    II_encode_bit_alloc(glb.bit_alloc, &glb.fr_ps, &glb.bs);
	    II_encode_scale(glb.bit_alloc, glb.scfsi, glb.scalar, &glb.fr_ps, &glb.bs);
	    II_subband_quantization(glb.scalar, *glb.sb_sample, glb.j_scale, *glb.j_sample, glb.bit_alloc, *glb.subband, &glb.fr_ps);
	    II_sample_encoding(*glb.subband, glb.bit_alloc, &glb.fr_ps, &glb.bs);

		for (i=0;i<glb.adb;i++)
			put1bit(&glb.bs, 0);

		#ifdef ADD_ANCILLARY_DATA
		// For J.52 bitrates there is no additional ancillary data
		if(glb.bitrate != 124800 && glb.bitrate != 62400)
		{
			UINT temp;
			static nValue=256;

			nValue+=15;

			nMaxRightEnergy=min(nMaxRightEnergy,0x7FF0);
			nMaxLeftEnergy=min(nMaxLeftEnergy,0x7FF0);

			/* values seems to be normalized between 0..10000 */
			nMaxRightEnergy=(INT)(nMaxRightEnergy/3.2768);
			nMaxLeftEnergy=(INT)(nMaxLeftEnergy/3.2768);


			if (glb.fr_ps.stereo==2)
			{
				temp=(nMaxRightEnergy>>8);
				putbits(&glb.bs,temp,8);

				temp=(nMaxRightEnergy&0xFF);
				putbits(&glb.bs,temp,8);
			}

			temp=0x00;
			putbits(&glb.bs,temp,8);

			temp=(nMaxLeftEnergy>>8);
			putbits(&glb.bs,temp,8);
			temp=(nMaxLeftEnergy&0xFF);
			putbits(&glb.bs,temp,8);
		}
#endif
		{
			UINT temp = 0;
			for(i = 0; i < glb.j52ancillaryzeros; i++)
				putbits(&glb.bs, temp, 8);
		}

#endif
	    break;
	    
/***************************** Layer 3 **********************************/

	  case 3:
	  {
#ifndef LAYER3
		MPegError("Layer 3 not supported\n");
#else
	      // large "auto" vars are static due to the Macintosh linker

			static double xr[2][2][576];
	      static double xr_dec[2][2][576];
	      static double pe[2][2];
	      static int l3_enc[2][2][576];
	      static III_psy_ratio ratio;
	      static III_side_info_t l3_side;
	      static III_scalefac_t  scalefac;
	      int gr, mode_gr, ch;
	      int mean_bits, sideinfo_len;
	      
	      int bitsPerFrame = 8 * glb.whole_SpF + (glb.fr_ps.header->padding * 8);
	      mode_gr = (glb.fr_ps.header->version == 1) ? 2 : 1;

	      // determine the mean bitrate for main data

		  sideinfo_len = 32;
	      if ( glb.fr_ps.header->version == 1 )
	      {
				// MPEG 1
				if ( glb.fr_ps.stereo == 1 )
					sideinfo_len += 136;
				else
					sideinfo_len += 256;
	      }
	      else
	      {
				// MPEG 2
				if ( glb.fr_ps.stereo == 1 )
					sideinfo_len += 72;
				else
					sideinfo_len += 136;
	      }

	      if ( glb.fr_ps.header->error_protection )
			sideinfo_len += 16;

	      mean_bits = (bitsPerFrame - sideinfo_len) / mode_gr;

	      // psychoacoustic model
	      for ( gr = 0; gr < mode_gr; gr++ )
			  for ( ch = 0; ch < glb.fr_ps.stereo; ch++ )
			  {
			      L3psycho_anal( &glb.buffer[ch][gr*576], &glb.sam[ch][0], ch,
				     glb.snr32, s_freq[glb.fr_ps.header->version][glb.fr_ps.header->sampling_frequency] * 1000.0,
				     &ratio.l[gr][ch][0], &ratio.s[gr][ch][0],
				     &pe[gr][ch], &l3_side.gr[gr].ch[ch].tt );
				}


		// polyphase filtering
		for( gr = 0; gr < mode_gr; gr++ )
		{
			for ( ch = 0; ch < glb.fr_ps.stereo; ch++ )
			{
				for ( j = 0; j < 18; j++ )
				{
//					window_subband( &glb.win_buf[ch], &(*glb.win_que)[ch][0], ch );
//					filter_subband( &(*glb.win_que)[ch][0],  &(*glb.l3_sb_sample)[ch][gr+1][j][0] );
					window_and_filter_subband(&glb.win_buf[ch],ch,&(*glb.l3_sb_sample)[ch][gr+1][j][0]);
		      }
			}
		}

	      // apply mdct to the polyphase outputs
	      mdct_sub( glb.l3_sb_sample, xr, glb.fr_ps.stereo, &l3_side, mode_gr );

		  // bit and noise allocation
	      iteration_loop( pe, xr, &ratio, &l3_side, l3_enc, mean_bits,
			      glb.fr_ps.stereo, xr_dec, &scalefac, &glb.fr_ps, 0, bitsPerFrame );

	      // write the frame to the bitstream
	      III_format_bitstream( bitsPerFrame, &glb.fr_ps, l3_enc, &l3_side, &scalefac, &glb.bs,
				    xr, NULL, 0 );
#endif
	  }
	    break;  // end of layer 3 case
	    

	} // end switch
	
	glb.frameBits = sstell( &glb.bs ) - glb.sentBits;

	if ( glb.frameBits % glb.bitsPerSlot )
	{
		// a program failure
		fprintf( stderr, "Sent %ld bits = %ld slots plus %ld\n",glb.frameBits, glb.frameBits/glb.bitsPerSlot,glb.frameBits%glb.bitsPerSlot );
	}

	glb.sentBits += glb.frameBits;

	// Set number of output bytes
	*pdwOutput=glb.bs.nOutBufPos;

	return BE_ERR_SUCCESSFUL;
}


BOOL APIENTRY DllMain(HANDLE hModule, 
                      DWORD  ul_reason_for_call, 
                      LPVOID lpReserved)
{

#ifdef DEBUG_ENCODER
	static int			iInit=0;

	if (iInit==0)
	{
		OutputDebugString("\n\nNew or Re Initialized\n");
		iInit++;
	}
	else
	{
		OutputDebugString("\n\nAlready Initialized\n");
	}
#endif

    switch( ul_reason_for_call ) {
    case DLL_PROCESS_ATTACH:
	break;
    case DLL_THREAD_ATTACH:
#ifdef DEBUG_ENCODER
		OutputDebugString("DLL_THREAD_ATTACH\n");
#endif
	break;
    case DLL_THREAD_DETACH:
#ifdef DEBUG_ENCODER
		OutputDebugString("DLL_THREAD_DETACH\n");
#endif
	break;
    case DLL_PROCESS_DETACH:
#ifdef DEBUG_ENCODER
		OutputDebugString("DLL_PROCESS_DETACH\n");
#endif
	break;
    }
    return TRUE;
}



BE_ERR	WINAPI CE_InitStream(PCE_CONFIG pbeConfig, PDWORD dwSamples, PDWORD dwBufferSize, PHBE_STREAM phbeStream)
{
	static BE_CONFIG beConfig;

	beConfig.dwConfig=pbeConfig->dwConfig;
	beConfig.format.mp3.dwSampleRate=pbeConfig->format.mp3.dwSampleRate;
	beConfig.format.mp3.byMode=pbeConfig->format.mp3.byMode;
	beConfig.format.mp3.wBitrate=pbeConfig->format.mp3.wBitrate;
	beConfig.format.mp3.bCRC=pbeConfig->format.mp3.bCRC;
	beConfig.format.mp3.bCopyright=pbeConfig->format.mp3.bCopyright;
	beConfig.format.mp3.bOriginal=pbeConfig->format.mp3.bOriginal;
	return beInitStream(&beConfig,dwSamples,dwBufferSize,phbeStream);
}


BE_ERR	WINAPI CE_EncodeChunk(HBE_STREAM hbeStream, DWORD nSamples, PSHORT pSamples, PBYTE pOutput, PDWORD pdwOutput)
{
	return beEncodeChunk(hbeStream,nSamples,pSamples,pOutput,pdwOutput);
}

BE_ERR	WINAPI CE_DeinitStream(HBE_STREAM hbeStream, PBYTE pOutput, PDWORD pdwOutput)
{
	return beDeinitStream(hbeStream,pOutput,pdwOutput);
}


BE_ERR	WINAPI CE_CloseStream(HBE_STREAM hbeStream)
{
	return beCloseStream(hbeStream);
}


VOID WINAPI CE_Version(PBE_VERSION pbeVersion)
{
	beVersion(pbeVersion);
}


#ifdef LAYER2
__declspec(dllexport) BE_ERR CE2_InitStream(PBE_CONFIG pbeConfig, PDWORD dwSamples, PDWORD dwBufferSize, PHBE_STREAM phbeStream)
{
	return beInitStream(pbeConfig,dwSamples,dwBufferSize,phbeStream);
}
__declspec(dllexport) BE_ERR CE2_EncodeChunk(HBE_STREAM hbeStream, DWORD nSamples, PSHORT pSamples, PBYTE pOutput, PDWORD pdwOutput)
{
	return beEncodeChunk(hbeStream,nSamples,pSamples,pOutput,pdwOutput);
}
__declspec(dllexport) BE_ERR CE2_DeinitStream(HBE_STREAM hbeStream, PBYTE pOutput, PDWORD pdwOutput)
{
	return beDeinitStream(hbeStream,pOutput,pdwOutput);
}
__declspec(dllexport) BE_ERR CE2_CloseStream(HBE_STREAM hbeStream)
{
	return beCloseStream(hbeStream);
}
__declspec(dllexport) VOID CE2_Version(PBE_VERSION pbeVersion)
{
	beVersion(pbeVersion);
}
#endif


#ifdef LAYER3
__declspec(dllexport) BE_ERR CE3_InitStream(PBE_CONFIG pbeConfig, PDWORD dwSamples, PDWORD dwBufferSize, PHBE_STREAM phbeStream)
{
	return beInitStream(pbeConfig,dwSamples,dwBufferSize,phbeStream);
}
__declspec(dllexport) BE_ERR CE3_EncodeChunk(HBE_STREAM hbeStream, DWORD nSamples, PSHORT pSamples, PBYTE pOutput, PDWORD pdwOutput)
{
	return beEncodeChunk(hbeStream,nSamples,pSamples,pOutput,pdwOutput);
}
__declspec(dllexport) BE_ERR CE3_DeinitStream(HBE_STREAM hbeStream, PBYTE pOutput, PDWORD pdwOutput)
{
	return beDeinitStream(hbeStream,pOutput,pdwOutput);
}
__declspec(dllexport) BE_ERR CE3_CloseStream(HBE_STREAM hbeStream)
{
	return beCloseStream(hbeStream);
}
__declspec(dllexport) VOID CE3_Version(PBE_VERSION pbeVersion)
{
	beVersion(pbeVersion);
}
#endif
