/**********************************************************************
 * ISO MPEG Audio Subgroup Software Simulation Group (1996)
 * ISO 13818-3 MPEG-2 Audio Encoder - Lower Sampling Frequency Extension
 *
 * $Id: encode.cpp,v 1.1.1.1 1999/12/05 21:15:49 afaber Exp $
 *
 * $Log: encode.cpp,v $
 * Revision 1.1.1.1  1999/12/05 21:15:49  afaber
 * no message
 *
 * Revision 1.2  1998/10/05 17:06:47  larsi
 * *** empty log message ***
 *
 * Revision 1.1.1.1  1998/10/05 14:47:17  larsi
 *
 * Revision 1.1  1996/02/14 04:04:23  rowlands
 * Initial revision
 *
 * Received from Mike Coleman
 **********************************************************************/

 
#include "common.h"
#include "encoder.h"
#include "subs.h"

enum byte_order NativeByteOrder = order_littleEndian;


// Local Function proto types
static void read_ana_window(double ana_win[HAN_SIZE]);
static void IDCT32(DOUBLE* xin,DOUBLE* xout );


/*=======================================================================\
|                                                                       |
| This segment contains all the core routines of the encoder,           |
| except for the psychoacoustic models.                                 |
|                                                                       |
| The user can select either one of the two psychoacoustic              |
| models. Model I is a simple tonal and noise masking threshold         |
| generator, and Model II is a more sophisticated cochlear masking      |
| threshold generator. Model I is recommended for lower complexity      |
| applications whereas Model II gives better subjective quality at low  |
| bit rates.                                                            |
|                                                                       |
| Layers I and II of mono, stereo, and joint stereo modes are supported.|
| Routines associated with a given layer are prefixed by "I_" for layer |
| 1 and "II_" for layer 2.                                              |
\=======================================================================*/
 
/************************************************************************
*
* read_samples()
*
* PURPOSE:  reads the PCM samples from a file to the buffer
*
*  SEMANTICS:
* Reads #samples_read# number of shorts from #musicin# filepointer
* into #sample_buffer[]#.  Returns the number of samples read.
*
************************************************************************/

#ifndef _BLADEDLL
ULONG read_samples(	FILE*	musicin,
					short sample_buffer[2304],
					ULONG num_samples,
					ULONG frame_size
					)
{
    ULONG samples_read;
    static ULONG samples_to_read;
    static char init = TRUE;
    extern int swapbytes; /* if TRUE then force swapping of byte order */

    if (init) {
        samples_to_read = num_samples;
        init = FALSE;
    }
    if (samples_to_read >= frame_size)
        samples_read = frame_size;
    else
        samples_read = samples_to_read;
    if ((samples_read =
         fread(sample_buffer, sizeof(short), (int)samples_read, musicin)) == 0)
      { /* printf("Hit end of audio data\n"); */ }
    /*
       Samples are big-endian. If this is a little-endian machine
       we must swap
     */
//    if ( NativeByteOrder == order_unknown )
//      {
//	NativeByteOrder = DetermineByteOrder();
//	if ( NativeByteOrder == order_unknown )
//	  {
//	    fprintf( stderr, "byte order not determined\n" );
//	    exit( 1 );
//	  }
//      }
    if (!iswav && ( NativeByteOrder == order_littleEndian ))
      SwapBytesInWords( sample_buffer, samples_read );

    if (swapbytes==TRUE)
      SwapBytesInWords( sample_buffer, samples_read );

    samples_to_read -= samples_read;
    if (samples_read < frame_size && samples_read > 0) {
      /* printf("Insufficient PCM input for one frame - fillout with zeros\n"); */
        for (; samples_read < frame_size; sample_buffer[samples_read++] = 0);
        samples_to_read = 0;
    }
    return(samples_read);
}
#endif

/************************************************************************
*
* get_audio()
*
* PURPOSE:  reads a frame of audio data from a file to the buffer,
*   aligns the data for future processing, and separates the
*   left and right channels
*
*
************************************************************************/
#ifndef _BLADEDLL 
ULONG get_audio(FILE* musicin,short buffer[2][1152],ULONG num_samples,int stereo,layer* info )
{
    int j;
    short insamp[3176];
    ULONG samples_read;
    int lay;
    extern int autoconvert;
    extern int resample;
    double resamplefactor;
    int temp1;

    resamplefactor=44.1/32.0;
    lay = info->lay;

    if ( (lay == 3) && (info->version == 0) )  /* ie MPEG-2 LSF */
    {
	if ( stereo == 2 )
	{
	    samples_read = read_samples( musicin, insamp, num_samples,
					 (ULONG) 1152 );
	    for ( j = 0; j < 576; j++ )
	    {
		buffer[0][j] = insamp[2 * j];
		buffer[1][j] = insamp[2 * j + 1];
	    }
	}
	else
	{
	    samples_read = read_samples( musicin, insamp, num_samples,
					 (ULONG) 576 );
	    for ( j = 0; j < 576; j++ )
	    {
		buffer[0][j] = insamp[j];
		buffer[1][j] = 0;
	    }
	}
    }
    else
    /* MPEG 1 */
      {
	    if(stereo == 2){ /* layer 2 (or 3), stereo */
	      if (resample==TRUE) {
		samples_read = read_samples(musicin, insamp, num_samples,
					    (ULONG) 3176);
		for(j=0;j<1152;j++) {
		    temp1=2*(int)((double)j*resamplefactor);
		    buffer[0][j] = insamp[temp1++];
		    buffer[1][j] = insamp[temp1];
		}
	      }
	      else {
		samples_read = read_samples(musicin, insamp, num_samples,
					    (ULONG) 2304);
		for(j=0;j<1152;j++) {
		    buffer[0][j] = insamp[2*j];
		    buffer[1][j] = insamp[2*j+1];
		}
	      }
	    }
	    else { /* layer 2 (or 3), mono */
	      if (autoconvert==TRUE) {  /* downconvert from a stereo file into a mono buffer */
		if (resample==TRUE) {
		  samples_read = read_samples(musicin, insamp, num_samples,
					      (ULONG) 3176);
		  for(j=0;j<1152;j++){
		    buffer[0][j] = insamp[2*(int)((double)j*resamplefactor)];
		    buffer[1][j] = 0;
		  }	
		}
		else {
		  samples_read = read_samples(musicin, insamp, num_samples,
					      (ULONG) 2304);
		  for(j=0;j<1152;j++){
		    buffer[0][j] = insamp[2*j];
		    buffer[1][j] = 0;
		  }	
		}	
	      }
	      else {
		samples_read = read_samples(musicin, insamp, num_samples,
					    (ULONG) 1152);
		for(j=0;j<1152;j++){
		    buffer[0][j] = insamp[j];
		    buffer[1][j] = 0;
		}
	      }
	    }
	}
    return(samples_read);
}
#endif
 

#ifdef ORG_SUBBANDSTUFF
/************************************************************************
*
* read_ana_window()
*
* PURPOSE:  Reads encoder window file "enwindow" into array #ana_win#
*
************************************************************************/


static void read_ana_window(double ana_win[HAN_SIZE])
{
  int i;
  for (i = 0; i<HAN_SIZE; i++) 
    ana_win[i] = enwindow[i];
}

/************************************************************************
*
* window_subband()
*
* PURPOSE:  Overlapping window on PCM samples
*
* SEMANTICS:
* 32 16-bit pcm samples are scaled to fractional 2's complement and
* concatenated to the end of the window buffer #x#. The updated window
* buffer #x# is then windowed by the analysis window #c# to produce the
* windowed sample #z#
*
************************************************************************/
 
void window_subband(short **pBuffer,double z[HAN_SIZE],int k)
{
	static double	x[2][HAN_SIZE];
	int				nWrap;
	static double*	pX;
	int				i,j;
	static int		off[2] = {0,0};
	static char		init = 0;
	static double	WindowFunc[HAN_SIZE];
	static double	dOneOverScale=1.0/SCALE;

	// First time around ?
    if (!init) 
	{
		// Clear memory of WindowFunc
		memset(WindowFunc,0x00,sizeof(double)*HAN_SIZE);
		
		// Read coefficients
        read_ana_window(WindowFunc);

		// Clear memory of x
		memset(x,0x00,sizeof(double)*2*HAN_SIZE);

		// Inidicate that we are done with initialization
        init = 1;
    }

	pX=&((x)[k][31-0+off[k]]);

    // replace 32 oldest samples with 32 new samples
    for (i=0;i<32;i++) 
	{
		*pX-- = (double) *(*pBuffer)++ *dOneOverScale;
	}

	pX=&((x)[k][0]);
	
	j=off[k];
	nWrap=HAN_SIZE-off[k];

    // shift samples into proper window positions
    for (i=0;i<HAN_SIZE;i++,j++) 
	{
		// Crossed the boundary, if so, adjust index
		if (i==nWrap)
			j-=HAN_SIZE;

		z[i] = pX[j] * WindowFunc[i];
	}

    off[k] += 480;              /*offset is modulo (HAN_SIZE-1)*/
    off[k] &= HAN_SIZE-1;
}
 
/************************************************************************
*
* filter_subband()
*
* PURPOSE:  Calculates the analysis filter bank coefficients
*
* SEMANTICS:
*      The windowed samples #z# is filtered by the digital filter matrix #m#
* to produce the subband samples #s#. This done by first selectively
* picking out values from the windowed samples, and then multiplying
* them by the filter matrix, producing 32 subband samples.
*
************************************************************************/

void create_dct_matrix(double filter[16][32])
{
  register int i,k;
 
  for (i=0; i<16; i++)
    for (k=0; k<32; k++) {
      if ((filter[i][k] = 1e9*cos((double)((2*i+1)*k*PI64))) >= 0)
	modf(filter[i][k]+0.5, &filter[i][k]);
      else
	modf(filter[i][k]-0.5, &filter[i][k]);
      filter[i][k] *= 1e-9;
    }
}

static void IDCT32(DOUBLE* xin,DOUBLE* xout )
{
	int i,j;
	double s0, s1;
	typedef double MM[16][32];
	static MM * m = 0;

	if( m==0 )
	{
		m = (MM*)mem_alloc(sizeof(MM),"filter");
		create_dct_matrix( *m );
	}

	for( i=0; i<16; i++ )
	{
		s0 = s1 = 0.0;
		for( j=0; j<32; j+=2 )
		{
			s0 += (*m)[i][j]*xin[j+0];
			s1 += (*m)[i][j+1]*xin[j+1];
		}
		xout[i] = s0+s1;
		xout[31-i] = s0-s1;
	}
}


void filter_subband(double z[HAN_SIZE],double s[SBLIMIT])
{
	double y[64],yprime[32];
	int i;
	double *zi;

	zi = z;
#if 1
	for( i=0; i<64; i++ )
	{
		y[ i ] = *zi + zi[ 64 ] + zi[ 128 ] + zi[ 192 ] +
		zi[ 256 ] + zi[ 320 ] + zi[ 384 ] + zi[ 448 ];
		zi++;
	}
#else

	for( i=0; i<64; i++ )
		y[i]=0;

	for( i=0; i<64*7; i++ )
	{
		y[i%64]+= *zi;
		zi++;
	}
#endif

	{
		yprime[0] = y[16];
		for( i=1; i<=16; i++ )
			yprime[i] = y[i+16]+y[16-i];

		for( i=17; i<=31; i++ ) yprime[i] = y[i+16]-y[80-i];
			IDCT32( yprime, s );
	}
}

#else

/************************************************************************
*
* filter_subband()
*
* PURPOSE:  Calculates the analysis filter bank coefficients
*
* SEMANTICS:
*      The windowed samples #z# is filtered by the digital filter matrix #m#
* to produce the subband samples #s#. This done by first selectively
* picking out values from the windowed samples, and then multiplying
* them by the filter matrix, producing 32 subband samples.
*
************************************************************************/

void create_dct_matrix(double filter[16][32])
{
  register int i,k;
 
  for (i=0; i<16; i++)
    for (k=0; k<32; k++) {
      if ((filter[i][k] = 1e9*cos((double)((2*i+1)*k*PI64))) >= 0)
	modf(filter[i][k]+0.5, &filter[i][k]);
      else
	modf(filter[i][k]-0.5, &filter[i][k]);
      filter[i][k] *= 1e-9;
    }
}

static void IDCT32(DOUBLE* xin,DOUBLE* xout )
{
	register int	i;
	register int	j;
	static int		bInit=0;
	DOUBLE			s0;
	DOUBLE			s1;
	static DOUBLE	m[16][32];

	if( !bInit )
	{
		create_dct_matrix( m );
		bInit++;
	}

	for( i=0; i<16; i++ )
	{
		s0 = s1 = 0.0;

//#define DONOTUNROLLIDCT
#ifdef DONOTUNROLLIDCT
		for( j=0; j<32; j+=2 )
		{
			s0 += m[i][j]*xin[j+0];
			s1 += m[i][j+1]*xin[j+1];
		}
#else
		s0 += m[i][ 0]*xin[ 0];
		s1 += m[i][ 1]*xin[ 1];
		s0 += m[i][ 2]*xin[ 2];
		s1 += m[i][ 3]*xin[ 3];
		s0 += m[i][ 4]*xin[ 4];
		s1 += m[i][ 5]*xin[ 5];
		s0 += m[i][ 6]*xin[ 6];
		s1 += m[i][ 7]*xin[ 7];
		s0 += m[i][ 8]*xin[ 8];
		s1 += m[i][ 9]*xin[ 9];
		s0 += m[i][10]*xin[10];
		s1 += m[i][11]*xin[11];
		s0 += m[i][12]*xin[12];
		s1 += m[i][13]*xin[13];
		s0 += m[i][14]*xin[14];
		s1 += m[i][15]*xin[15];
		s0 += m[i][16]*xin[16];
		s1 += m[i][17]*xin[17];
		s0 += m[i][18]*xin[18];
		s1 += m[i][19]*xin[19];
		s0 += m[i][20]*xin[20];
		s1 += m[i][21]*xin[21];
		s0 += m[i][22]*xin[22];
		s1 += m[i][23]*xin[23];
		s0 += m[i][24]*xin[24];
		s1 += m[i][25]*xin[25];
		s0 += m[i][26]*xin[26];
		s1 += m[i][27]*xin[27];
		s0 += m[i][28]*xin[28];
		s1 += m[i][29]*xin[29];
		s0 += m[i][30]*xin[30];
		s1 += m[i][31]*xin[31];
#endif
		xout[i] = s0+s1;
		xout[31-i] = s0-s1;
	}
}

// Window and filter subband
//
// Step 1:
//		Put new samples into input buffer (32 samples)
//		thus:
//			// Shift samples down 32 samples
//		    for (i=511;i>=32;i--) 
//			{
//				pdInputFifo[nChannel][i]=pdInputFifo[nChannel][i-32];
//			}
//		    for (;i>=0;i--) 
//			{
//				// Put in new data samples
//				pdInputFifo[nChannel][i  ]=(double)*(*pBuffer)++ *dOneOverScale;
//			}
// Step 2: 
//		Calculate z[i] and store directly in y[i%64]
//		thus:
//		
//			//clear y array
//			memset(y,0,64*sizeof(double));
//
//			// multiplt buffered input with window, add directly to y[i]
//		    for (i=0;i<HAN_SIZE;i++) 
//			{
//				// Apply hanning window
//				y[i%64]+=pdInputFifo[nChannel][i] * enwindow[i];
//			}
// Step 3:
//		Multiply with DFT matrix 
//
//			for (i=0;i<SBLIMIT;i++)
//			{
//				for (j=0, s[i]= 0;j<64;j++)
//				{
//					s[i] += (*m)[i][j] * y[j];
//				}
//		}
//		m[i][j]=cos((2*i+1)*(j-16)*PI/64)
// That is all
//

void window_and_filter_subband(short **pBuffer,int nChannel,double s[SBLIMIT])
{
	static double	y[64],yprime[32];
	register int	i,j;
	static double	pdInputFifo[2][HAN_SIZE];
	const double	dOneOverScale=1.0/SCALE;
	static int		off[2] = {0,0};

	//clear y array
	memset(y,0,64*sizeof(double));

    // replace 32 oldest samples with 32 new samples 
    for (i=31;i>=0;i--) 
	{
		pdInputFifo[nChannel][i+off[nChannel]] = (double)*(*pBuffer)++*dOneOverScale;
	}

    // Shift samples into proper window positions, and multiply with energy window
	i=0;
    while(i<HAN_SIZE) 
	{
		y[i%64]+=pdInputFifo[nChannel][(i+off[nChannel])&HAN_SIZE-1] * enwindow[i];i++;
		y[i%64]+=pdInputFifo[nChannel][(i+off[nChannel])&HAN_SIZE-1] * enwindow[i];i++;
		y[i%64]+=pdInputFifo[nChannel][(i+off[nChannel])&HAN_SIZE-1] * enwindow[i];i++;
		y[i%64]+=pdInputFifo[nChannel][(i+off[nChannel])&HAN_SIZE-1] * enwindow[i];i++;
	}

	// offset is modulo (HAN_SIZE-1)
    off[nChannel] += 480;
    off[nChannel] &= HAN_SIZE-1;

	// Due to symetries, thus stuff is optimized
	// See A high performance software implementation of MPEG Audio Encoder
	// Manoj Kumar and Mohammad Zubair
	// 0-7803-3192-3/96

	yprime[0] = y[16];

//#define DONOTUNROLL
#ifdef DONOTUNROLL1
	for( i=1; i<=16; i++ )
		yprime[i] = y[i+16]+y[16-i];
#else
	yprime[ 1] = y[17]+y[15];
	yprime[ 2] = y[18]+y[14];
	yprime[ 3] = y[19]+y[13];
	yprime[ 4] = y[20]+y[12];
	yprime[ 5] = y[21]+y[11];
	yprime[ 6] = y[22]+y[10];
	yprime[ 7] = y[23]+y[ 9];
	yprime[ 8] = y[24]+y[ 8];
	yprime[ 9] = y[25]+y[ 7];
	yprime[10] = y[26]+y[ 6];
	yprime[11] = y[27]+y[ 5];
	yprime[12] = y[28]+y[ 4];
	yprime[13] = y[29]+y[ 3];
	yprime[14] = y[30]+y[ 2];
	yprime[15] = y[31]+y[ 1];
	yprime[16] = y[32]+y[ 0];
#endif

#ifdef DONOTUNROLL1
	for( i=17; i<=31; i++ )
		yprime[i] = y[i+16]-y[80-i];
#else
	yprime[17] = y[33]-y[63];
	yprime[18] = y[34]-y[62];
	yprime[19] = y[35]-y[61];
	yprime[20] = y[36]-y[60];
	yprime[21] = y[37]-y[59];
	yprime[22] = y[38]-y[58];
	yprime[23] = y[39]-y[57];
	yprime[24] = y[40]-y[56];
	yprime[25] = y[41]-y[55];
	yprime[26] = y[42]-y[54];
	yprime[27] = y[43]-y[53];
	yprime[28] = y[44]-y[52];
	yprime[29] = y[45]-y[51];
	yprime[30] = y[46]-y[50];
	yprime[31] = y[47]-y[49];
#endif

	IDCT32( yprime, s );
}


#endif




#if (defined(LAYER2) || defined(LAYER1))
/************************************************************************
* encode_info()
*
* PURPOSE:  Puts the syncword and header information on the output
* bitstream.
*
************************************************************************/
 
void encode_info(frame_params *fr_ps,Bit_stream_struc *bs)
{
	layer *info = fr_ps->header;

	putbits(bs,0xfff,12);                    // syncword 12 bits
	put1bit(bs,info->version);               // ID        1 bit
	putbits(bs,4-info->lay,2);               // layer     2 bits
	put1bit(bs,!info->error_protection);     // bit set => no err prot
	putbits(bs,info->bitrate_index,4);
	putbits(bs,info->sampling_frequency,2);
	put1bit(bs,info->padding);
	put1bit(bs,info->extension);             // private_bit
	putbits(bs,info->mode,2);
	putbits(bs,info->mode_ext,2);
	put1bit(bs,info->copyright);
	put1bit(bs,info->original);
	putbits(bs,info->emphasis,2);
}
#endif

 
/************************************************************************
*
* mod()
*
* PURPOSE:  Returns the absolute value of its argument
*
************************************************************************/
 
_inline double mod(double a)
{
    return (a > 0) ? a : -a;
}
 

 
/************************************************************************
*
* encode_CRC
*
************************************************************************/
 
void encode_CRC(UINT crc,Bit_stream_struc* bs)
{
   putbits(bs, crc, 16);
}




#ifdef LAYER2


/******************************** Layer II ******************************/
 
void II_scale_factor_calc(	double	sb_sample[][3][SCALE_BLOCK][SBLIMIT],
							UINT	scalar[][3][SBLIMIT],
							int		stereo,
							int		sblimit)
{
  int i,j, k,t;
  double s[SBLIMIT];
 
  for (k=0;k<stereo;k++) for (t=0;t<3;t++) {
    for (i=0;i<sblimit;i++)
      for (j=1, s[i] = mod(sb_sample[k][t][0][i]);j<SCALE_BLOCK;j++)
        if (mod(sb_sample[k][t][j][i]) > s[i])
             s[i] = mod(sb_sample[k][t][j][i]);
 
  for (i=0;i<sblimit;i++)
    for (j=SCALE_RANGE-2,scalar[k][t][i]=0;j>=0;j--)    /* $A 6/16/92 */
      if (s[i] <= multiple[j]) {
         scalar[k][t][i] = j;
         break;
      }
      for (i=sblimit;i<SBLIMIT;i++) scalar[k][t][i] = SCALE_RANGE-1;
    }
}

/************************************************************************
*
* pick_scale  (Layer II)
*
* PURPOSE:For each subband, puts the smallest scalefactor of the 3
* associated with a frame into #max_sc#.  This is used
* used by Psychoacoustic Model I.
* (I would recommend changin max_sc to min_sc)
*
************************************************************************/
 
void pick_scale(	UINT scalar[2][3][SBLIMIT],
					frame_params *fr_ps,
					double max_sc[2][SBLIMIT]
				)
{
  int i,j,k;
  UINT	max;
  int stereo  = fr_ps->stereo;
  int sblimit = fr_ps->sblimit;
 
  for (k=0;k<stereo;k++)
    for (i=0;i<sblimit;max_sc[k][i] = multiple[max],i++)
      for (j=1, max = scalar[k][0][i];j<3;j++)
         if (max > scalar[k][j][i]) max = scalar[k][j][i];
  for (i=sblimit;i<SBLIMIT;i++) max_sc[0][i] = max_sc[1][i] = 1E-20;
}


/************************************************************************
*
* II_transmission_pattern (Layer II only)
*
* PURPOSE:For a given subband, determines whether to send 1, 2, or
* all 3 of the scalefactors, and fills in the scalefactor
* select information accordingly
*
* SEMANTICS:  The subbands and channels are classified based on how much
* the scalefactors changes over its three values (corresponding
* to the 3 sets of 12 samples per subband).  The classification
* will send 1 or 2 scalefactors instead of three if the scalefactors
* do not change much.  The scalefactor select information,
* #scfsi#, is filled in accordingly.
*
************************************************************************/
 
void II_transmission_pattern(
								UINT	scalar[2][3][SBLIMIT],
								UINT	scfsi[2][SBLIMIT],
								frame_params *fr_ps
							)
{
   int stereo  = fr_ps->stereo;
   int sblimit = fr_ps->sblimit;
   int dscf[2];
   int nClass[2],i,j,k;
static int pattern[5][5] = {0x123, 0x122, 0x122, 0x133, 0x123,
                            0x113, 0x111, 0x111, 0x444, 0x113,
                            0x111, 0x111, 0x111, 0x333, 0x113,
                            0x222, 0x222, 0x222, 0x333, 0x123,
                            0x123, 0x122, 0x122, 0x133, 0x123};
 
   for (k=0;k<stereo;k++)
     for (i=0;i<sblimit;i++) {
       dscf[0] =  (scalar[k][0][i]-scalar[k][1][i]);
       dscf[1] =  (scalar[k][1][i]-scalar[k][2][i]);
       for (j=0;j<2;j++) {
         if (dscf[j]<=-3) nClass[j] = 0;
         else if (dscf[j] > -3 && dscf[j] <0) nClass[j] = 1;
              else if (dscf[j] == 0) nClass[j] = 2;
                   else if (dscf[j] > 0 && dscf[j] < 3) nClass[j] = 3;
                        else nClass[j] = 4;
       }
       switch (pattern[nClass[0]][nClass[1]]) {
         case 0x123 :    scfsi[k][i] = 0;
                         break;
         case 0x122 :    scfsi[k][i] = 3;
                         scalar[k][2][i] = scalar[k][1][i];
                         break;
         case 0x133 :    scfsi[k][i] = 3;
                         scalar[k][1][i] = scalar[k][2][i];
                         break;
         case 0x113 :    scfsi[k][i] = 1;
                         scalar[k][1][i] = scalar[k][0][i];
                         break;
         case 0x111 :    scfsi[k][i] = 2;
                         scalar[k][1][i] = scalar[k][2][i] = scalar[k][0][i];
                         break;
         case 0x222 :    scfsi[k][i] = 2;
                         scalar[k][0][i] = scalar[k][2][i] = scalar[k][1][i];
                         break;
         case 0x333 :    scfsi[k][i] = 2;
                         scalar[k][0][i] = scalar[k][1][i] = scalar[k][2][i];
                         break;
         case 0x444 :    scfsi[k][i] = 2;
                         if (scalar[k][0][i] > scalar[k][2][i])
                              scalar[k][0][i] = scalar[k][2][i];
                         scalar[k][1][i] = scalar[k][2][i] = scalar[k][0][i];
      }
   }
}


/************************************************************************
*
* II_encode_scale (Layer II)
*
* PURPOSE:The encoded scalar factor information is arranged and
* queued into the output fifo to be transmitted.
*
* For Layer II, the three scale factors associated with
* a given subband and channel are transmitted in accordance
* with the scfsi, which is transmitted first.
*
************************************************************************/

void II_encode_scale(
						UINT				bit_alloc[2][SBLIMIT],
						UINT				scfsi[2][SBLIMIT],
						UINT				scalar[2][3][SBLIMIT],
						frame_params*		fr_ps,
						Bit_stream_struc*	bs
					)
{
   int stereo  = fr_ps->stereo;
   int sblimit = fr_ps->sblimit;
   int jsbound = fr_ps->jsbound;
   int i,j,k;
 
   for (i=0;i<sblimit;i++) for (k=0;k<stereo;k++)
     if (bit_alloc[k][i])  putbits(bs,scfsi[k][i],2);
 
   for (i=0;i<sblimit;i++) for (k=0;k<stereo;k++)
     if (bit_alloc[k][i])  /* above jsbound, bit_alloc[0][i] == ba[1][i] */
        switch (scfsi[k][i]) {
           case 0: for (j=0;j<3;j++)
                     putbits(bs,scalar[k][j][i],6);
                   break;
           case 1:
           case 3: putbits(bs,scalar[k][0][i],6);
                   putbits(bs,scalar[k][2][i],6);
                   break;
           case 2: putbits(bs,scalar[k][0][i],6);
        }
}
 
/************************************************************************
*
* I_bits_for_nonoise  (Layer I)
* II_bits_for_nonoise (Layer II)
*
* PURPOSE:Returns the number of bits required to produce a
* mask-to-noise ratio better or equal to the noise/no_noise threshold.
*
* SEMANTICS:
* bbal = # bits needed for encoding bit allocation
* bsel = # bits needed for encoding scalefactor select information
* banc = # bits needed for ancillary data (header info included)
*
* For each subband and channel, will add bits until one of the
* following occurs:
* - Hit maximum number of bits we can allocate for that subband
* - MNR is better than or equal to the minimum masking level
*   (NOISY_MIN_MNR)
* Then the bits required for scalefactors, scfsi, bit allocation,
* and the subband samples are tallied (#req_bits#) and returned.
*
* (NOISY_MIN_MNR) is the smallest MNR a subband can have before it is
* counted as 'noisy' by the logic which chooses the number of JS
* subbands.
*
* Joint stereo is supported.
*
************************************************************************/

static double snr[18] = {0.00, 7.00, 11.00, 16.00, 20.84,
                         25.28, 31.59, 37.75, 43.84,
                         49.89, 55.93, 61.96, 67.98, 74.01,
                         80.03, 86.05, 92.01, 98.01};

int II_bits_for_nonoise(double perm_smr[2][SBLIMIT],UINT scfsi[2][SBLIMIT],frame_params *fr_ps)
{
   int sb,ch,ba;
   int stereo  = fr_ps->stereo;
   int sblimit = fr_ps->sblimit;
   int jsbound = fr_ps->jsbound;
   al_table *alloc = fr_ps->alloc;
   int req_bits = 0, bbal = 0, berr = 0, banc = 32;
   int maxAlloc, sel_bits, sc_bits, smp_bits;
static int sfsPerScfsi[] = { 3,2,1,2 };    /* lookup # sfs per scfsi */

   /* added 92-08-11 shn */
   if (fr_ps->header->error_protection) berr=16; else berr=0; 
 
   for (sb=0; sb<jsbound; ++sb)
     bbal += stereo * (*alloc)[sb][0].bits;
   for (sb=jsbound; sb<sblimit; ++sb)
     bbal += (*alloc)[sb][0].bits;
   req_bits = banc + bbal + berr;
 
   for(sb=0; sb<sblimit; ++sb)
     for(ch=0; ch<((sb<jsbound)?stereo:1); ++ch) {
       maxAlloc = (1<<(*alloc)[sb][0].bits)-1;
       sel_bits = sc_bits = smp_bits = 0;
       for(ba=0;ba<maxAlloc-1; ++ba)
         if( (-perm_smr[ch][sb] + snr[(*alloc)[sb][ba].quant+((ba>0)?1:0)])
             >= NOISY_MIN_MNR)
            break;      /* we found enough bits */
       if(stereo == 2 && sb >= jsbound) /* check other JS channel */
         for(;ba<maxAlloc-1; ++ba)
           if( (-perm_smr[1-ch][sb]+ snr[(*alloc)[sb][ba].quant+((ba>0)?1:0)])
               >= NOISY_MIN_MNR)
             break;
       if(ba>0) {
         smp_bits = SCALE_BLOCK * ((*alloc)[sb][ba].group * (*alloc)[sb][ba].bits);
         /* scale factor bits required for subband */
         sel_bits = 2;
         sc_bits  = 6 * sfsPerScfsi[scfsi[ch][sb]];
         if(stereo == 2 && sb >= jsbound) {
           /* each new js sb has L+R scfsis */
           sel_bits += 2;
           sc_bits  += 6 * sfsPerScfsi[scfsi[1-ch][sb]];
         }
         req_bits += smp_bits+sel_bits+sc_bits;
       }
   }
   return req_bits;
}
 
/************************************************************************
*
* II_main_bit_allocation  (Layer II)
*
* PURPOSE:For joint stereo mode, determines which of the 4 joint
* stereo modes is needed.  Then calls *_a_bit_allocation(), which
* allocates bits for each of the subbands until there are no more bits
* left, or the MNR is at the noise/no_noise threshold.
*
* SEMANTICS:
*
* For joint stereo mode, joint stereo is changed to stereo if
* there are enough bits to encode stereo at or better than the
* no-noise threshold (NOISY_MIN_MNR).  Otherwise, the system
* iteratively allocates less bits by using joint stereo until one
* of the following occurs:
* - there are no more noisy subbands (MNR >= NOISY_MIN_MNR)
* - mode_ext has been reduced to 0, which means that all but the
*   lowest 4 subbands have been converted from stereo to joint
*   stereo, and no more subbands may be converted
*
*     This function calls *_bits_for_nonoise() and *_a_bit_allocation().
*
************************************************************************/

void II_main_bit_allocation(double perm_smr[2][SBLIMIT], UINT scfsi[2][SBLIMIT],UINT bit_alloc[2][SBLIMIT],INT* adb,frame_params* fr_ps)
{
   int  noisy_sbs;
   int  mode, mode_ext, lay;
   int  rq_db, av_db = *adb;
 
   if((mode = fr_ps->actual_mode) == MPG_MD_JOINT_STEREO) {
     fr_ps->header->mode = MPG_MD_STEREO;
     fr_ps->header->mode_ext = 0;
     fr_ps->jsbound = fr_ps->sblimit;
     if((rq_db=II_bits_for_nonoise(perm_smr, scfsi, fr_ps)) > *adb) {
       fr_ps->header->mode = MPG_MD_JOINT_STEREO;
       mode_ext = 4;           /* 3 is least severe reduction */
       lay = fr_ps->header->lay;
       do {
         --mode_ext;
         fr_ps->jsbound = js_bound(lay, mode_ext);
         rq_db = II_bits_for_nonoise(perm_smr, scfsi, fr_ps);
       } while( (rq_db > *adb) && (mode_ext > 0));
       fr_ps->header->mode_ext = mode_ext;
     }    /* well we either eliminated noisy sbs or mode_ext == 0 */
   }
   noisy_sbs = II_a_bit_allocation(perm_smr, scfsi, bit_alloc, adb, fr_ps);
}
 
/************************************************************************
*
* II_a_bit_allocation (Layer II)
*
* PURPOSE:Adds bits to the subbands with the lowest mask-to-noise
* ratios, until the maximum number of bits for the subband has
* been allocated.
*
* SEMANTICS:
* 1. Find the subband and channel with the smallest MNR (#min_sb#,
*    and #min_ch#)
* 2. Calculate the increase in bits needed if we increase the bit
*    allocation to the next higher level
* 3. If there are enough bits available for increasing the resolution
*    in #min_sb#, #min_ch#, and the subband has not yet reached its
*    maximum allocation, update the bit allocation, MNR, and bits
    available accordingly
* 4. Repeat until there are no more bits left, or no more available
*    subbands. (A subband is still available until the maximum
*    number of bits for the subband has been allocated, or there
*    aren't enough bits to go to the next higher resolution in the
    subband.)
*
************************************************************************/

int II_a_bit_allocation(double perm_smr[2][SBLIMIT],UINT scfsi[2][SBLIMIT],UINT bit_alloc[2][SBLIMIT],int *adb,frame_params *fr_ps)
{
   int i, min_ch, min_sb, oth_ch, k, increment, scale, seli, ba;
   int bspl, bscf, bsel, ad, noisy_sbs, bbal=0;
   double mnr[2][SBLIMIT];
   double dSmall;
   char used[2][SBLIMIT];
   int stereo  = fr_ps->stereo;
   int sblimit = fr_ps->sblimit;
   int jsbound = fr_ps->jsbound;
   al_table *alloc = fr_ps->alloc;
static char init= 0;
static int banc=32, berr=0;
static int sfsPerScfsi[] = { 3,2,1,2 };    /* lookup # sfs per scfsi */
 
   if (!init) { 
       init = 1;  
       if (fr_ps->header->error_protection) berr=16; /* added 92-08-11 shn */
   }
   for (i=0; i<jsbound; ++i)
     bbal += stereo * (*alloc)[i][0].bits;
   for (i=jsbound; i<sblimit; ++i)
     bbal += (*alloc)[i][0].bits;
   *adb -= bbal + berr + banc;
   ad = *adb;
 
   for (i=0;i<sblimit;i++) for (k=0;k<stereo;k++) {
     mnr[k][i]=snr[0]-perm_smr[k][i];
     bit_alloc[k][i] = 0;
     used[k][i] = 0;
   }
   bspl = bscf = bsel = 0;
 
   do  {
     /* locate the subband with minimum SMR */
     dSmall = 999999.0; min_sb = -1; min_ch = -1;
     for (i=0;i<sblimit;i++) for(k=0;k<stereo;++k)
       if (used[k][i]  != 2 && dSmall > mnr[k][i]) {
         dSmall = mnr[k][i];
         min_sb = i;  min_ch = k;
     }
     if(min_sb > -1) {   /* there was something to find */
       /* find increase in bit allocation in subband [min] */
       increment = SCALE_BLOCK * ((*alloc)[min_sb][bit_alloc[min_ch][min_sb]+1].group *
                        (*alloc)[min_sb][bit_alloc[min_ch][min_sb]+1].bits);
       if (used[min_ch][min_sb])
         increment -= SCALE_BLOCK * ((*alloc)[min_sb][bit_alloc[min_ch][min_sb]].group*
                           (*alloc)[min_sb][bit_alloc[min_ch][min_sb]].bits);
 
       /* scale factor bits required for subband [min] */
       oth_ch = 1 - min_ch;    /* above js bound, need both chans */
       if (used[min_ch][min_sb]) scale = seli = 0;
       else {          /* this channel had no bits or scfs before */
         seli = 2;
         scale = 6 * sfsPerScfsi[scfsi[min_ch][min_sb]];
         if(stereo == 2 && min_sb >= jsbound) {
           /* each new js sb has L+R scfsis */
           seli += 2;
           scale += 6 * sfsPerScfsi[scfsi[oth_ch][min_sb]];
         }
       }
       /* check to see enough bits were available for */
       /* increasing resolution in the minimum band */
       if (ad >= bspl + bscf + bsel + seli + scale + increment) {
         ba = ++bit_alloc[min_ch][min_sb]; /* next up alloc */
         bspl += increment;  /* bits for subband sample */
         bscf += scale;      /* bits for scale factor */
         bsel += seli;       /* bits for scfsi code */
         used[min_ch][min_sb] = 1; /* subband has bits */
         mnr[min_ch][min_sb] = -perm_smr[min_ch][min_sb] +
                               snr[(*alloc)[min_sb][ba].quant+1];
         /* Check if subband has been fully allocated max bits */
         if (ba >= (1<<(*alloc)[min_sb][0].bits)-1) used[min_ch][min_sb] = 2;
       }
       else used[min_ch][min_sb] = 2; /* can't increase this alloc */
       if(min_sb >= jsbound && stereo == 2) {
         /* above jsbound, alloc applies L+R */
         ba = bit_alloc[oth_ch][min_sb] = bit_alloc[min_ch][min_sb];
         used[oth_ch][min_sb] = used[min_ch][min_sb];
         mnr[oth_ch][min_sb] = -perm_smr[oth_ch][min_sb] +
                               snr[(*alloc)[min_sb][ba].quant+1];
       }
     }
   } while(min_sb > -1);   /* until could find no channel */
   /* Calculate the number of bits left */
   ad -= bspl+bscf+bsel;   *adb = ad;
   for (i=sblimit;i<SBLIMIT;i++) for (k=0;k<stereo;k++) bit_alloc[k][i]=0;
 
   noisy_sbs = 0;  dSmall = mnr[0][0];      /* calc worst noise in case */
   for(k=0;k<stereo;++k) {
     for (i=0;i<sblimit;i++) {
       if (dSmall > mnr[k][i]) dSmall = mnr[k][i];
       if(mnr[k][i] < NOISY_MIN_MNR) ++noisy_sbs; /* noise is not masked */

     }
   }
   return noisy_sbs;
}
 
/************************************************************************
*
* II_subband_quantization (Layer II)
*
* PURPOSE:Quantizes subband samples to appropriate number of bits
*
* SEMANTICS:  Subband samples are divided by their scalefactors, which
 makes the quantization more efficient. The scaled samples are
* quantized by the function a*x+b, where a and b are functions of
* the number of quantization levels. The result is then truncated
* to the appropriate number of bits and the MSB is inverted.
*
* Note that for fractional 2's complement, inverting the MSB for a
 negative number x is equivalent to adding 1 to it.
*
************************************************************************/
 
static double a[17] = {
  0.750000000, 0.625000000, 0.875000000, 0.562500000, 0.937500000,
  0.968750000, 0.984375000, 0.992187500, 0.996093750, 0.998046875,
  0.999023438, 0.999511719, 0.999755859, 0.999877930, 0.999938965,
  0.999969482, 0.999984741 };
 
static double b[17] = {
  -0.250000000, -0.375000000, -0.125000000, -0.437500000, -0.062500000,
  -0.031250000, -0.015625000, -0.007812500, -0.003906250, -0.001953125,
  -0.000976563, -0.000488281, -0.000244141, -0.000122070, -0.000061035,
  -0.000030518, -0.000015259 };
 
void II_subband_quantization(	UINT scalar[2][3][SBLIMIT],
								double sb_samples[2][3][SCALE_BLOCK][SBLIMIT],
								UINT j_scale[3][SBLIMIT],
								double j_samps[3][SCALE_BLOCK][SBLIMIT],
								UINT bit_alloc[2][SBLIMIT],
								UINT sbband[2][3][SCALE_BLOCK][SBLIMIT],
								frame_params *fr_ps)
{
   int		i, j, k, s, n, qnt, sig;
   int		stereo  = fr_ps->stereo;
   int		sblimit = fr_ps->sblimit;
   int		jsbound = fr_ps->jsbound;
   UINT		stps;
   double	d;

   al_table *alloc = fr_ps->alloc;

   for (s=0;s<3;s++)
     for (j=0;j<SCALE_BLOCK;j++)
       for (i=0;i<sblimit;i++)
         for (k=0;k<((i<jsbound)?stereo:1);k++)
           if (bit_alloc[k][i]) {
             /* scale and quantize floating point sample */
             if(stereo == 2 && i>=jsbound)       /* use j-stereo samples */
               d = j_samps[s][j][i] / multiple[j_scale[s][i]];
             else
               d = sb_samples[k][s][j][i] / multiple[scalar[k][s][i]];
             if (mod(d) > 1.0)
               printf("Not scaled properly %d %d %d %d\n",k,s,j,i);
             qnt = (*alloc)[i][bit_alloc[k][i]].quant;
             d = d * a[qnt] + b[qnt];
             /* extract MSB N-1 bits from the floating point sample */
             if (d >= 0) sig = 1;
             else { sig = 0; d += 1.0; }
             n = 0;
#ifndef MS_DOS
             stps = (*alloc)[i][bit_alloc[k][i]].steps;
             while ((1<<n) < (int)stps) n++;
#else
             while  ( ( (ULONG)(1L<<(long)n) <
                       ((ULONG) ((*alloc)[i][bit_alloc[k][i]].steps)
                        & 0xffff
                        )
                       ) && ( n <16)
                     ) n++;
#endif
             n--;
             sbband[k][s][j][i] = (UINT) (d * (double) (1L<<n));
             /* tag the inverted sign bit to sbband at position N */
             /* The bit inversion is a must for grouping with 3,5,9 steps
                so it is done for all subbands */
             if (sig) sbband[k][s][j][i] |= 1<<n;
           }
           for (s=0;s<3;s++)
             for (j=sblimit;j<SBLIMIT;j++)
               for (i=0;i<SCALE_BLOCK;i++) for (k=0;k<stereo;k++) sbband[k][s][i][j] = 0;
}
 
/*************************************************************************
* II_encode_bit_alloc (Layer II)
*
* PURPOSE:Writes bit allocation information onto bitstream
*
* Layer I uses 4 bits/subband for bit allocation information,
* and Layer II uses 4,3,2, or 0 bits depending on the
* quantization table used.
*
************************************************************************/
 
void II_encode_bit_alloc(
							UINT bit_alloc[2][SBLIMIT],
							frame_params *fr_ps,
							Bit_stream_struc *bs
						)
{
   int i,k;
   int stereo  = fr_ps->stereo;
   int sblimit = fr_ps->sblimit;
   int jsbound = fr_ps->jsbound;
   al_table *alloc = fr_ps->alloc;
 
   for (i=0;i<sblimit;i++)
     for (k=0;k<((i<jsbound)?stereo:1);k++)
       putbits(bs,bit_alloc[k][i],(*alloc)[i][0].bits);
}
 
/************************************************************************
*
* II_sample_encoding  (Layer II)
*
* PURPOSE:Put one frame of subband samples on to the bitstream
*
* SEMANTICS:  The number of bits allocated per sample is read from
* the bit allocation information #bit_alloc#.  Layer 2
* supports writing grouped samples for quantization steps
* that are not a power of 2.
*
************************************************************************/

void II_sample_encoding(	UINT sbband[2][3][SCALE_BLOCK][SBLIMIT],
							UINT bit_alloc[2][SBLIMIT],
							frame_params *fr_ps,
							Bit_stream_struc *bs
						)
{
   UINT temp;
   UINT j,s,x,y;
   int		i,k;
   int stereo  = fr_ps->stereo;
   int sblimit = fr_ps->sblimit;
   int jsbound = fr_ps->jsbound;
   al_table *alloc = fr_ps->alloc;
 
   for (s=0;s<3;s++)
     for (j=0;j<SCALE_BLOCK;j+=3)
       for (i=0;i<sblimit;i++)
         for (k=0;k<((i<jsbound)?stereo:1);k++)
           if (bit_alloc[k][i]) {
             if ((*alloc)[i][bit_alloc[k][i]].group == 3) {
               for (x=0;x<3;x++) putbits(bs,sbband[k][s][j+x][i],
                                         (*alloc)[i][bit_alloc[k][i]].bits);
             }
             else {
               y =(*alloc)[i][bit_alloc[k][i]].steps;
               temp = sbband[k][s][j][i] +
                      sbband[k][s][j+1][i] * y +
                      sbband[k][s][j+2][i] * y * y;
               putbits(bs,temp,(*alloc)[i][bit_alloc[k][i]].bits);
             }
           }
}



void II_combine_LR(double	sb_sample[2][3][SCALE_BLOCK][SBLIMIT],
				   double	joint_sample[3][SCALE_BLOCK][SBLIMIT],
				   int		sblimit)
{  
	// make a filtered mono for joint stereo
	int sb, smp, sufr;

	for(sb = 0; sb<sblimit; ++sb)
		for(smp = 0; smp<SCALE_BLOCK; ++smp)
			for(sufr = 0; sufr<3; ++sufr)
				joint_sample[sufr][smp][sb] = .5 * (sb_sample[0][sufr][smp][sb]
										   + sb_sample[1][sufr][smp][sb]);
}


#endif