/**********************************************************************
 * ISO MPEG Audio Subgroup Software Simulation Group (1996)
 * ISO 13818-3 MPEG-2 Audio Encoder - Lower Sampling Frequency Extension
 *
 * $Id: encoder.h,v 1.1.1.1 1999/12/05 21:15:49 afaber Exp $
 *
 * $Log: encoder.h,v $
 * Revision 1.1.1.1  1999/12/05 21:15:49  afaber
 * no message
 *
 * Revision 1.1  1996/02/14 04:04:23  rowlands
 * Initial revision
 *
 * Received from Mike Coleman
 **********************************************************************/

#ifndef ENCODER_DOT_H
#define ENCODER_DOT_H

#include "common.h"
#include "tables.h"


/***********************************************************************
*
*  Encoder Include Files
*
***********************************************************************/

/***********************************************************************
*
*  Encoder Definitions
*
***********************************************************************/

/* General Definitions */

/* Default Input Arguments (for command line control) */

#ifdef LAYER3
#define DFLT_LAY        3      /* default encoding layer is II */
#endif
#ifdef LAYER2
#define DFLT_LAY        2      /* default encoding layer is II */
#endif

#define DFLT_MOD        's'    /* default mode is stereo */
#define DFLT_PSY        2      /* default psych model is 1 */
#define DFLT_SFQ        44.1   /* default input sampling rate is 44.1 kHz */
#define DFLT_EMP        'n'    /* default de-emphasis is none */

#ifdef LAYER2
#define DFLT_EXT        ".mp2" /* default output file extension */
#endif
#ifdef LAYER3
#define DFLT_EXT        ".mp3" /* default output file extension */
#endif

#define FILETYPE_ENCODE 'TEXT'
#define CREATOR_ENCODE  'MpgD'

// This is the smallest MNR a subband can have before it is counted
// as 'noisy' by the logic which chooses the number of JS subbands

#define NOISY_MIN_MNR   0.0

// Psychacoustic Model 1 Definitions

#define CB_FRACTION     0.33
#define MAX_SNR         1000
#define NOISE           10
#define TONE            20
#define DBMIN           -200.0
#define LAST            -1
#define STOP            -100
#define POWERNORM       90.3090 /* = 20 * log10(32768) to normalize */
                                /* max output power to 96 dB per spec */

// Psychoacoustic Model 2 Definitions

#define LOGBLKSIZE      10
#define BLKSIZE         1024
#define HBLKSIZE        513
#define CBANDS          63
#define LXMIN           32.0

/***********************************************************************
*
*  Encoder Type Definitions
*
***********************************************************************/

/* Psychoacoustic Model 1 Type Definitions */

typedef int        IFFT2[FFT_SIZE/2];
typedef int        IFFT[FFT_SIZE];
typedef double     D9[9];
typedef double     D10[10];
typedef double     D640[640];
typedef double     D1408[1408];
typedef double     DFFT2[FFT_SIZE/2];
typedef double     DFFT[FFT_SIZE];
typedef double     DSBL[SBLIMIT];
typedef double     D2SBL[2][SBLIMIT];

/*
typedef struct {
        int        line;
        double     bark, hear, x;
} g_thres, *g_ptr;
*/
typedef struct {
        double     x;
        int        type, next, map;
} mask, *mask_ptr;

// Psychoacoustic Model 2 Type Definitions

typedef int        ICB[CBANDS];
typedef int        IHBLK[HBLKSIZE];
typedef FLOAT      F32[32];
typedef FLOAT      F2_32[2][32];
typedef FLOAT      FCB[CBANDS];
typedef FLOAT      FCBCB[CBANDS][CBANDS];
typedef FLOAT      FBLK[BLKSIZE];
typedef FLOAT      FHBLK[HBLKSIZE];
typedef FLOAT      F2HBLK[2][HBLKSIZE];
typedef FLOAT      F22HBLK[2][2][HBLKSIZE];
typedef double     MYDCB[CBANDS];

extern int iswav;






/***********************************************************************
*
*  Encoder Function Prototype Declarations
*
***********************************************************************/


//void filter_subband(double z[HAN_SIZE],double s[SBLIMIT]);


#if (defined(LAYER2) || defined(LAYER1))

void psycho_anal(	SHORT*	buffer,
					SHORT	savebuf[1056],
					INT		chn,
					INT		lay,
					FLOAT	snr32[32],
					DOUBLE	sfreq
				);

void   encode_info(frame_params*, Bit_stream_struc*);
#endif

// The following functions are in the file "encode.c"

extern unsigned long    read_samples(FILE*, short[2304], unsigned long,
                           unsigned long);
#if 0
extern unsigned long    get_audio(FILE*, short[2][1152], unsigned long,
                           int, int);
#else
extern unsigned long    get_audio(FILE*, short[2][1152], unsigned long,
                           int, layer* info);
#endif
//void   window_subband(short**, double[HAN_SIZE], int);
//void   create_ana_filter(double[SBLIMIT][64]);
void window_and_filter_subband(short **pBuffer,int nChannel,double s[SBLIMIT]);


double mod(double);
void   I_combine_LR(double[2][3][SCALE_BLOCK][SBLIMIT],
                           double[3][SCALE_BLOCK][SBLIMIT]);
void   I_scale_factor_calc(double[][3][SCALE_BLOCK][SBLIMIT],
                           UINT[][3][SBLIMIT], int);
void   pick_scale(UINT[2][3][SBLIMIT], frame_params*,
                           double[2][SBLIMIT]);
void   put_scale(UINT[2][3][SBLIMIT], frame_params*,
                           double[2][SBLIMIT]);
void   I_encode_scale(UINT[2][3][SBLIMIT],
                           UINT[2][SBLIMIT], frame_params*,
                           Bit_stream_struc*);
int    II_bits_for_nonoise(double[2][SBLIMIT], UINT[2][SBLIMIT],
                           frame_params*);
void   II_main_bit_allocation(double[2][SBLIMIT],
                           UINT[2][SBLIMIT], UINT[2][SBLIMIT],
                           int*, frame_params*);
int    II_a_bit_allocation(double[2][SBLIMIT], UINT[2][SBLIMIT],
                           UINT[2][SBLIMIT], int*, frame_params*);
int    I_bits_for_nonoise(double[2][SBLIMIT], frame_params*);
void   I_main_bit_allocation(double[2][SBLIMIT],
                           UINT[2][SBLIMIT], int*, frame_params*);
int    I_a_bit_allocation(double[2][SBLIMIT], UINT[2][SBLIMIT],
                           int*, frame_params*);
void   I_subband_quantization(UINT[2][3][SBLIMIT],
                           double[2][3][SCALE_BLOCK][SBLIMIT], UINT[3][SBLIMIT],
                           double[3][SCALE_BLOCK][SBLIMIT], UINT[2][SBLIMIT],
                           UINT[2][3][SCALE_BLOCK][SBLIMIT], frame_params*);
/*
extern void   II_subband_quantization(UINT[2][3][SBLIMIT],
                           double[2][3][SCALE_BLOCK][SBLIMIT], UINT[3][SBLIMIT],
                           double[3][SCALE_BLOCK][SBLIMIT], UINT[2][SBLIMIT],
                           UINT[2][3][SCALE_BLOCK][SBLIMIT], frame_params*);
*/
void II_subband_quantization(	UINT scalar[2][3][SBLIMIT],
								double sb_samples[2][3][SCALE_BLOCK][SBLIMIT],
								UINT j_scale[3][SBLIMIT],
								double j_samps[3][SCALE_BLOCK][SBLIMIT],
								UINT bit_alloc[2][SBLIMIT],
								UINT sbband[2][3][SCALE_BLOCK][SBLIMIT],
								frame_params *fr_ps);

void   II_encode_bit_alloc(UINT[2][SBLIMIT], frame_params*,Bit_stream_struc*);
void   I_encode_bit_alloc(UINT[2][SBLIMIT], frame_params*,Bit_stream_struc*);
void   I_sample_encoding(UINT[2][3][SCALE_BLOCK][SBLIMIT],UINT[2][SBLIMIT], frame_params*,Bit_stream_struc*);
void II_sample_encoding(	UINT sbband[2][3][SCALE_BLOCK][SBLIMIT],
							UINT bit_alloc[2][SBLIMIT],
							frame_params *fr_ps,
							Bit_stream_struc *bs
						);

void   encode_CRC(UINT, Bit_stream_struc*);


#ifdef LAYER2
void II_combine_LR(double	sb_sample[2][3][SCALE_BLOCK][SBLIMIT],
				   double	joint_sample[3][SCALE_BLOCK][SBLIMIT],
				   int		sblimit);

void II_scale_factor_calc(	double	sb_sample[][3][SCALE_BLOCK][SBLIMIT],
							UINT	scalar[][3][SBLIMIT],
							int		stereo,
							int		sblimit);

void II_transmission_pattern(
								UINT	scalar[2][3][SBLIMIT],
								UINT	scfsi[2][SBLIMIT],
								frame_params *fr_ps
							);

void II_encode_scale(
						UINT				bit_alloc[2][SBLIMIT],
						UINT				scfsi[2][SBLIMIT],
						UINT				scalar[2][3][SBLIMIT],
						frame_params*		fr_ps,
						Bit_stream_struc*	bs
					);

#endif


#endif
