#ifndef TYPES_INCLUDED
#define TYPES_INCLUDED

#include <stdio.h>
#include <stdlib.h>
#include <direct.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <assert.h>


#ifdef _WIN32
	#include <windows.h>
	#define DOUBLE double
#else
	#undef UINT
	#undef FLOAT
	#define  UINT unsigned int 
	#define FLOAT float
	#define DOUBLE double
	#define SHORT	short
#endif

#define MPEGFRAMESIZE 1152


enum byte_order { order_unknown, order_bigEndian, order_littleEndian };


#define PROTO_ARGS

#ifndef EXIT_SUCCESS
	#define EXIT_SUCCESS 0
#endif

#ifndef EXIT_FAILURE
	#define EXIT_FAILURE 1
#endif

/***********************************************************************
*
*  Global Definitions
*
***********************************************************************/

#ifndef FALSE
	#define         FALSE                   0
#endif

#ifndef TRUE
	#define         TRUE                    1
#endif

#define         NULL_CHAR               '\0'

#define         MAX_U_32_NUM            0xFFFFFFFF
#ifndef PI
	#define         PI                      3.14159265358979
#endif
#define         PI4                     PI/4
#define         PI64                    PI/64
#define         LN_TO_LOG10             0.2302585093

#define         VOL_REF_NUM             0
#define         MPEG_AUDIO_ID           1
#define			MPEG_PHASE2_LSF			0		// Low Sample frequencies MPEG-2
#define         MAC_WINDOW_SIZE         24

#define         MONO                    1
#define         STEREO                  2
#define         BITS_IN_A_BYTE          8
//#define         WORD                    16
#define         MAX_NAME_SIZE           81
#define         SBLIMIT                 32
#define         SSLIMIT                 18
#define         FFT_SIZE                1024
#define         HAN_SIZE                512
#define         SCALE_BLOCK             12
#define         SCALE_RANGE             64
#define         SCALE                   32768
#define         CRC16_POLYNOMIAL        0x8005

// MPEG Header Definitions - Mode Values
#define         MPG_MD_STEREO           0
#define         MPG_MD_JOINT_STEREO     1
#define         MPG_MD_DUAL_CHANNEL     2
#define         MPG_MD_MONO             3

// Mode Extention
#define         MPG_MD_LR_LR             0
#define         MPG_MD_LR_I              1
#define         MPG_MD_MS_LR             2
#define         MPG_MD_MS_I              3


// "bit_stream.h" Definitions
#define         MINIMUM         4    // Minimum size of the buffer in bytes
#define         MAX_LENGTH      32   // Maximum length of word written or read from bit stream
                                        
#define         READ_MODE       0
#define         WRITE_MODE      1
#define         ALIGNING        8
#define         BINARY          0
#define         ASCII           1

#ifndef BS_FORMAT
#define         BS_FORMAT       BINARY	// BINARY or ASCII = 2x bytes
#endif

#define         BUFFER_SIZE     8192

#define         MIN(A, B)       ((A) < (B) ? (A) : (B))
#define         MAX(A, B)       ((A) > (B) ? (A) : (B))

/***********************************************************************
*
*  Global Type Definitions
*
***********************************************************************/


// Header Information Structure
typedef struct
{
    int version;
    int lay;
    int error_protection;
    int bitrate_index;
    int sampling_frequency;
    int padding;
    int extension;
    int mode;
    int mode_ext;
    int copyright;
    int original;
    int emphasis;
} layer, *the_layer;

// Parent Structure Interpreting some Frame Parameters in Header
// "bit_stream.h" Type Definitions

typedef struct  bit_stream_struc
{
//#ifdef _BLADEDLL
	BYTE*		pbtOutBuf;
	int			nOutBufPos;
//#else
    FILE        *pt;            // pointer to bit stream device
//#endif
    unsigned char *buf;         // bit stream buffer
    int         buf_size;       // size of buffer (in number of bytes)
    long        totbit;         // bit counter of bit stream
    int         buf_byte_idx;   // pointer to top byte in buffer
    int         buf_bit_idx;    // pointer to top bit of top byte in buffer
    int         mode;           // bit stream open in read or write mode
    int         eob;            // end of buffer index
    int         eobs;           // end of bit stream flag
    char        format;
    // format of file in rd mode (BINARY/ASCII)
} Bit_stream_struc;


#ifdef  WIN32
	#define         TABLES_PATH     "tables\\"  // to find data files
#endif

#ifdef  UNIX
	// to find data files
	#define         TABLES_PATH     "tables"  
	// name of environment variable holding path of table files
	#define         MPEGTABENV      "MPEGTABLES"
	// how to build paths
	#define         PATH_SEPARATOR  "/"        
#endif  // UNIX

typedef struct {
	unsigned int    steps;
	unsigned int    bits;
	unsigned int    group;
	unsigned int    quant;
} sb_alloc, *alloc_ptr;

typedef sb_alloc        al_table[SBLIMIT][16];


typedef struct {
    layer       *header;        // raw header information
    int         actual_mode;    // when writing IS, may forget if 0 chs
    al_table    *alloc;         // bit allocation table read in
    int         tab_num;        // number of table as loaded
    int         stereo;         // 1 for mono, 2 for stereo
    int         jsbound;        // first band of joint stereo coding
    int         sblimit;        // total number of sub bands
} frame_params;

#ifdef LAYER2

	// Structure for Reading Layer II Allocation Tables from File
	typedef struct 
	{
			int        dummy;
			int        line;
			double     bark, hear, x;
	} g_thres, *g_ptr;


	typedef struct ALLOCTABLE_TAG
	{
		int				i,j;
		sb_alloc		alloc;
	} ALLOCTABLE;

#endif
#endif
